
# q-expansion of newform 448.8.a.t, downloaded from the LMFDB on 01 June 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField, ZZ
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K([c/ZZ(den) for c in num]) for num, den in basis_data]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 448
weight = 8
poly_data = [-216, -1, 1]

# The entries in the following list give a basis for the
# coefficient ring in terms of a root of the defining polynomial above.
# Each line consists of the coefficients of the numerator, and a denominator.
basis_data  = [[[1, 0], 1], [[-1, 2], 1]]

hecke_ring_character_values = None
aps_data = [[0, 0], [47, -1], [-165, 5], [343, 0], [1422, 58], [-1267, -441], [-744, -162], [16405, 429], [3288, 24], [-10320, -4690], [195918, 1254], [-183696, 13590], [367332, 11858], [-240238, 2814], [544554, 18450], [-1429422, -2592], [80085, -26811], [432323, -28779], [-164324, -25464], [3750108, -65436], [2150622, -95964], [3204220, 138540], [5829537, 66297], [4886130, -181592], [5381376, 47334], [-6389157, -219323], [1881778, -568542], [11560656, 164980], [-1210140, 415602], [8134212, -607166], [21572724, -376068], [11655837, -1100363], [15335286, 276684], [6569115, 1075011], [7403310, 250428], [-483932, 264612], [7635749, 1620879], [-42187598, 1093782], [-10797066, 1723526], [-24221607, -1421597], [-80409720, 930932], [62213333, -287385], [31484388, -1398604], [23293712, 674046], [-13292946, 1664348], [16072590, 2612142], [26830372, -4082400], [54349148, 6659100], [149746581, 3933789], [30409585, 6344931], [242877042, 1734744], [123934200, -6249936], [-85733608, 4378254], [-208949013, -1647821], [135771066, 4160312], [350704668, 6839684], [299220105, 3301299], [-9775672, 208584], [136633874, -4999248], [-112633638, -8658160], [405902677, 6981141], [232922823, -9607871], [337616461, -14555499], [-402862752, -8262120], [-22328428, 6304818], [160730694, 7327260], [106333922, 11815374], [68237416, 26956902], [-278864214, 24283814], [-545757415, -9261], [-144704478, 7965040], [-160298700, 36624052], [-673523436, 4761156], [-218156402, 19773372], [-831217770, 15004290], [182368938, -4987614], [-1571609880, -21551154], [-125908311, -38344749], [-1090448448, 7535406], [-361488640, -41574114], [33435675, -12885285], [-385132022, -44161320], [1013223168, 12210504], [-365296148, -85402590], [-2658629880, 15679536], [18139452, 68295504], [2102318010, 8968344], [-2205723224, 44507838], [538176933, -83299265], [1742307488, -1054032], [539677401, -48666399], [995168190, -48922250], [16435544, -46498776], [-177833388, -87650248], [2941702780, 56720808], [2105500068, -24420228], [-2736594375, 34026739], [-1144498308, -158168550], [1869770137, -50928855], [1095837118, -115889148], [-116828234, 74713674], [-4648381746, 32697908], [-4359504753, -11256449], [-1795419120, -107190842], [2292125102, -26889798], [1016776906, -44666280], [-1098382089, 193030159], [4231984782, -97421364], [1849900800, 248197080], [2941593522, 73461192], [4753933264, 35799216], [-4323799432, -150467634], [-2177895984, 10416294], [-577984955, 214669461], [-3974664332, -68055204], [3105337992, -23480826], [-9570025633, 39528447], [-742168866, 154365982], [-2636187612, -57456606], [-8939316930, -161451486], [-6012304177, 114965145], [4751941462, -169576092], [-4965930771, 100918095], [-3183615588, 271939824], [-4760559163, 94458765], [8603090508, -122137638], [-1084550280, 379676286], [-3629408010, 149460750], [-8132379206, -173368566], [9440700363, -348461955], [6446309010, 25537206], [-16563895812, 168749532], [4463096048, 541092408], [13624761904, -98923146], [12892059912, 121927542], [1551198740, -112216566], [-3037556457, 592589425], [-17347321679, -14955879], [14851808769, 148153019], [-7789028400, 407305446], [14500460177, 99641745], [673923738, -258200744], [-9284703072, -49731768], [9096820896, 524363700], [-3799382165, -136493475], [-18019487430, 340737866], [19276830433, -447536061], [19978524816, 90672910], [-4892518165, -420925965], [-1008706932, 726556092], [15654586904, -470772810], [-9752787618, 494122860], [-12654791728, 298871916], [-24983767806, 353220082], [12073828376, 59289708], [-3005505372, -345510900], [-11555326400, 214936872], [14847428880, 787236326], [-17813031474, 38774484], [-14861782437, -442076963], [-6543027594, 517859738], [22832896242, -57324568], [-7356763696, 1027399968], [-2087411763, 1159192365], [3630801966, -1341966708], [7522234218, -670212150], [28705747048, -423436680], [-47824639441, -5329911], [-31125303200, -881668578], [4752198153, -1250279669], [23639116740, 1029656952], [-27516610107, 139025103], [20175701628, 465699300], [32753148002, 624579720], [22165632190, 68685366], [-23735545890, 118294820], [-7887546728, -363933108], [57122717448, -454847458], [-30543924852, 206027556], [5689923515, 1101213405], [-14229547376, -983890296], [-29430126873, -111786297], [10307181988, 1045547034], [-5112783294, -1554400552], [13452153588, 1601549724], [26148664845, 1227720607], [-44788580446, -550491384], [56497959397, -238544979], [-48551667050, -1240531764], [29506811658, 1934093330], [20859407482, -701455032], [41633376612, -1835470056], [-9647885728, -1910247300], [-29708204187, 795865551], [489614616, 3409447204], [-53679911658, 1710759084], [-19827630808, -2055929370], [-48871535232, 1520663742], [31213815486, -2337113604], [95248128, -758534184], [28998674970, -1752579152], [49699255018, 2070210114], [-14911429761, -1422509559], [-24567917810, -732514644], [53296278945, -1669860367], [14273915259, 1344018061], [4856210250, -2428912110], [45078520482, 2149860150], [-34432851780, 216207474], [-26936112233, 1708355487], [-15505776984, 2515838070], [17640038283, 1839006309], [23413145348, 1604247876], [-38831695899, -2072923067], [-24873078480, -4778480888], [3593854072, 4692239190], [88823864464, -1846870944], [58662906972, -1938293918], [-29831243136, 2591129432], [-72363149532, -633693774], [49294643788, -126715278], [42094537520, 2692801272], [27879231690, -1239964792], [60922534888, 3152699616], [-50899737819, -1862657923], [-53849602340, 1795218642], [103517855802, -1566427280], [26969593440, 3122607976], [-167356319876, -474168060], [149124104502, 1769347458], [-25235818252, 1013571186], [134989764355, 948695187], [-18056655752, -5639201736], [13333019442, -1303932488], [45525571497, -5586665175], [-13469097696, -303745056], [-103500345560, -3593404722], [63444289158, 1044771892], [-140182847460, -1887811632], [31118609508, -6895912100], [183076304622, 328757634], [-81967619583, -270641439], [-3264746532, 3153567012], [83798537450, -2942482848], [167688955182, 3061742460], [-74226873690, 1229192438], [-221476759336, -669175752], [190218320997, -3759849283], [44421734050, -503207010], [118969300728, 323743224], [164921024014, 1361859156], [31352163882, 3111183000], [20991048924, 580980020], [-121908683750, -2823049152], [18595675443, 882268661], [-20894431920, 2581923740], [-160795328492, 492260970], [-149421374769, 5260254999], [81685942779, 4138114533], [23532419096, -2483366058], [277507949368, -499456512], [-249017622054, 1699830062], [30133325035, -6478042059], [-160031021717, -4179619395], [116336515776, -5063718682], [211487446475, 595363995], [164841745710, 3173455908], [167487234642, 7298413192], [-155231744288, 713639220], [-40543233732, 4474378802], [-153308938087, -5239239357], [155627658446, 2750655042], [150885356662, -256866660], [-6085340900, -4622914524], [-40196531774, 6285473040], [-84949851452, -9610215540], [-78798205134, -7449152546], [-113074455810, 11563945524], [-112100422628, 1028383122], [-230193308103, -7221538735], [342170887788, 1606527532], [-62349084792, 12556256928], [7116754554, 10623499978], [66049207343, 11865457065], [-419852383639, 4647124617], [-263244660492, 1548559260], [-395006970518, -3276321480], [129361001334, -7303083564], [-87480650150, 11614184394], [-142945613790, -8442525168], [-64116326142, -10027939336], [-244901109399, 7594182049], [490246114752, -2176021242], [-6831404223, 12157741281], [-52800437902, -4446093960], [3891030705, 10745306027], [-403187201512, 316437000], [307934665773, 2235405823], [-548555691645, -3441913333], [-88467571129, 11515680855], [11754278232, 10877321478], [-111229923156, -1917371334], [193045198520, -8129800632], [62367382002, 17749216534], [147810842202, 3522893574], [-65423383674, -13315697388], [96171556986, -6192421338], [-78911876505, 2531594457], [537546054960, -3367275328], [16629506738, -19332981504], [-240655549572, -6074949876], [-12250210680, 19202477950], [60426887502, 7553149692], [-99503138958, 10854014502], [43667660244, -1740154388], [531929001770, -5501100816], [-47967730545, 11621919519], [-258411046272, -9519736704], [-506774342018, -2230695012], [3571624170, 14162335320], [-299793006613, 9833523867], [-70518406754, -21849138972], [-409148986197, 8107068669], [152019649948, 14282863644], [282697540392, 5479546494], [-505672350788, -8848530990], [60495762070, -9512342934], [-455584152943, -7102244511], [-127010463876, 4590327220], [215013400908, 16188642834], [-111193447402, -4839462660], [-366109500996, 11346376074], [164502742810, -18492603822], [858905670747, -2630704245], [329100745912, 22675366044], [745200923481, -11516197511], [-190227582260, 4217036586], [-132330445278, 18906686640], [610959174602, -9933051072], [-111439125886, 4193320794], [553424505878, -4135144428], [537311286696, 19845959294], [563334993165, 9124704687], [558454349108, -11443811988], [244556966148, -8006794478], [-79237353690, -9167510870], [-60424463587, 23429390559], [491578965166, -12283833558], [-363639456012, -12711029436], [567609461079, -19854205919], [529788181927, -12931401705], [420327319266, 6352625736], [406396346298, -8817849040], [-37019034582, 8724183714], [175974173030, 23914495524], [-828835861098, -4402440868], [-155024287170, -11070853194], [218598878517, 25621040661], [20360119026, -29059738816], [-237570415392, 19233590208], [34380604494, 2144744364], [173921461662, -32766958460], [-430265572836, -26292759180], [-46712839080, 28907471996], [-38142360979, -236618859], [240499554472, -12305576706], [701691199209, 3638561395], [-752142747112, 14357037000], [-193161144888, 37671852078], [713550949392, -15332846436], [84202498755, 17549590347], [-681597557832, -28010740776], [828511179270, -15388508940], [-674628140072, -4720906224], [-18499017826, 22648778148], [684201497565, 3376216885], [463837689768, 33269998160], [-647998974898, -8587598580], [997676907000, 10120212174], [-642837541584, 25839899022], [879220814313, -4932586301], [834215980992, 16801966902], [-276600125776, 8831074392], [1349521196346, -7190186344], [-605054252145, 24669632463], [-1662943021752, -1589044064], [-109678038212, 37541056236], [1079149264979, 4963384077], [18172055372, -51544316736], [-1049596787526, 18719209026], [-562060144280, -11806792074], [-413322603117, 36749417661], [-695537695320, -32166462900], [-127644011029, 14502764187], [-194333220132, 17321661052], [168236337382, 13996892940], [-215429941690, -46144644930], [-146414642220, -36050528430], [403020878472, 24052626444], [1111568584608, -20567843130], [-954370044485, -27116041395], [624723357162, -12959029240], [-43054317616, -26001871872], [588526287696, 16969164942], [410459035905, -13955812653], [-98894913982, -9431294574], [180189290994, -56191603848], [1047617589552, -41083816690], [-145723687573, -27446142501], [637092325335, -21329540937], [-1406723182378, -6311519772], [-416472181596, 36898435066], [-194731924098, 33146829002], [477211638892, 37703002152], [878057481786, 6065653440], [203980956693, 37680994671], [1139468509380, 44789901348], [-975760842446, 48740229042], [-992734702344, 38147839758], [374983327428, -58300582020], [503077162020, -1692942078], [207122252039, 69536929089], [-59523416136, -949136552], [454141659615, 32347167671], [-93622304188, 27035144850], [1539189832119, 39577428297], [-738201513138, -13985452190], [275423845650, -24849604248], [-334065302433, 43381032183], [-420426861930, -52980331242]]
